.report-description
{
  margin-top: 0px;
  margin-left: 20px;
  padding-top: 0px;
 }

.criteria {
    margin-bottom: 10px;
}

.criteria-group-title
{
    font-weight: bold;
    background-color: #E0E0E0;
    padding: 5px;
    margin-bottom: 10px;
}

.date { width: 100px; }
.time { width: 100px; }
.duration { width: 100px; }
.filterexpr { width: 300px; }

#reports-dropdown {
    position: relative;
    float: right;
    top: 0px;
    height: 26px;
}

.widget-dropdown-menu {
    left: auto;
    right: 0px;
    text-align: left;
}

.widget-error-modal pre {
    font-size: 10px;
}

.widget-error {
    font-size: 13px;
}

.wid-outer-container {
    overflow: hidden;
    width: 100%;
    height: 100%;
}

.wid-title {
    height: 20px;
    text-align: center;
}

.wid-content {
    margin: 10px;
}

#print-report {
    float: right;
    margin-top: 2px;
    margin-right: 1px;
    margin-left: 4px;
}

/* When the print report icon is a span (rather than a link), gray it out */
span#print-report {
    opacity: 0.4;
}

.modal-content { /* All modal content divs hidden initially */
    display: none;
}

.modal .modal-content { /* But shown if they're inside a modal */
    display: block;
}

/* Inputs for embed widget dimensions */
#embed-widget-width, #embed-widget-height {
    width: 50px;
    margin-top: 10px;
}

#embed-widget-code {
    width: 97%;
}


input[type="file"] {
    padding-top: 5px;
    line-height: 0;
}
