/**
 # Copyright (c) 2013 Riverbed Technology, Inc.
 #
 # This software is licensed under the terms and conditions of the
 # MIT License set forth at:
 #   https://github.com/riverbed/flyscript-portal/blob/master/LICENSE ("License").
 # This software is distributed "AS IS" as set forth in the License.
 */

@media print
{
  div
  { page-break-inside: avoid; }
}

.blackbox
{
  border: solid 1px black;
  margin-bottom: 10px;
}

.login-box
{
  text-align: center;
  width: 100%;
  max-width: 500px;
}

.login-box .span9
{
  text-align: center;
  margin-top: 5px;
  margin-bottom: 5px;
}

.login-box .span3
{
  text-align: right;
  padding-top: 4px;
  margin-top: 5px;
  margin-bottom: 5px;
}

.login-box .span12
{
  text-align: center;
  padding: 10px;
}

.login-box .input-large
{
  width:100%;
  max-width: 230px;
}

.widget-box
{
  border: solid 1px black;
  margin-bottom: 10px;
}

.widget-title
{
  font-size: 110%;
  font-weight: bold;
  padding: 5px;
  background-color: #F0F0F0;
}

.widget-content
{
  border-top: solid 1px black;
  padding: 5px;
}

.block-centered
{
    margin-left: auto !important;
    margin-right: auto !important;
    margin-top: 20px !important;
    margin-bottom: 20px ! important;
    float: none !important;
}

.bordered
{
    border: 1px solid #DADADA;
    padding: 10px;
}

.row-disabled
{
    color: #b4b4b4;
}

/* MAPS */

#map_canvas img { max-width: none; }

div.circleMarker {
    background-color: #ff0000;
    border-color: #ff0000;
    border-radius: 999px;
    border-style: solid;
    border-width: 1px;
    opacity: 0.5;
}

/* MESSAGES */

ul#messages {
    margin: 0 0 10px 0;
    padding: 0;
    list-style: none;
}
ul#messages li {
    padding: 5px 10px;
    background: #EFD;
    margin: 3px;
    border: 1px solid #BC9;
}
ul#messages li a {
    float: right;
    color: #693;
}

