{% extends "site_base.html" %}
{% block comments %}
<!--
# Copyright (c) 2014 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.
-->
{% endblock %}


{% load url from future %}
{% block login %}{% endblock %}

{% block content %}

<div class="container-fluid">
    <div class="row-fluid">
        <div class="login-box block-centered">
            <form class="form-horizontal well" method="post" action="{% url 'django.contrib.auth.views.login' %}">
                {% csrf_token %}
                <fieldset>
                    <legend>Welcome To SteelScript Application Framework</legend>
                    <div class="row-fluid">
                        <div class="span3">
                            <label>Username:</label>
                        </div>
                        <div class="span9">
                            <input id="id_username" name="username" type="text" placeholder="type your username" class="input-large">
                        </div>
                    </div>

                    <div class="row-fluid">
                        <div class="span3">
                            <label>Password:</label>
                        </div>
                        <div class="span9">
                            <input id="id_password" name="password" type="password" placeholder="type your password" class="input-large">
                        </div>
                        {% if form.errors %}
                            <p class="text-error text-center">Your username and/or password didn't match. Please try again.</p>
                        {% endif %}
                    </div>

                    <div class="row-fluid">
                        <div class="span12">
                            <button type="submit" id="submit" class="btn btn-primary button-loading" data-loading-text="Loading...">Sign in</button>
                        </div>
                    </div>
                </fieldset>

                <input type="hidden" name="next" value="{{ next }}" />
            </form>
        </div>
    </div>
</div>

<script type="text/javascript">document.getElementById("id_username").focus();</script>
{% endblock %}
